@@ -24,10 +24,17 @@
 #include <ags/plugin/ags_base_plugin.h>
 #include <ags/plugin/ags_plugin_port.h>
 
+#include <ags/audio/ags_playback_domain.h>
+#include <ags/audio/ags_playback.h>
 #include <ags/audio/ags_input.h>
 #include <ags/audio/ags_recall_container.h>
 #include <ags/audio/ags_port_util.h>
 
+#include <ags/audio/thread/ags_audio_thread.h>
+#include <ags/audio/thread/ags_channel_thread.h>
+
+#include <ags/audio/task/ags_write_vst3_port.h>
+
 #include <ags/audio/fx/ags_fx_vst3_channel.h>
 
 #include <ags/i18n.h>
@@ -432,9 +439,6 @@ ags_fx_vst3_audio_notify_samplerate_callback(GObject *gobject,
 
   GList *start_recall_channel, *recall_channel;
 
-  gchar **parameter_name;
-  
-  guint n_params;
   guint input_pads;
   guint audio_channels;
   guint output_port_count, input_port_count;
@@ -443,8 +447,6 @@ ags_fx_vst3_audio_notify_samplerate_callback(GObject *gobject,
   guint i, j, k;
   gboolean is_live_instrument;
 
-  GValue *value;
-  
   GRecMutex *recall_mutex;
   
   fx_vst3_audio = AGS_FX_VST3_AUDIO(gobject);
@@ -471,52 +473,6 @@ ags_fx_vst3_audio_notify_samplerate_callback(GObject *gobject,
     return;
   }
 
-#if HAVE_GLIB_2_68
-  strv_builder = g_strv_builder_new();
-
-  g_strv_builder_add(strv_builder,
-		     "buffer-size");
-  g_strv_builder_add(strv_builder,
-		     "samplerate");
-  g_strv_builder_add(strv_builder,
-		     "iedit-controller");
-  g_strv_builder_add(strv_builder,
-		     "iaudio-processor");
-  g_strv_builder_add(strv_builder,
-		     "iedit-controller-host-editing");
-  
-  parameter_name = g_strv_builder_end(strv_builder);
-#else
-  parameter_name = (gchar **) g_malloc(6 * sizeof(gchar *));
-
-  parameter_name[0] = g_strdup("buffer-size");
-  parameter_name[1] = g_strdup("samplerate");
-  parameter_name[2] = g_strdup("iedit-controller");
-  parameter_name[3] = g_strdup("iaudio-processor");
-  parameter_name[4] = g_strdup("iedit-controller-host-editing");
-  parameter_name[5] = NULL;
-#endif
-
-  n_params = 5;
-
-  value = g_new0(GValue,
-		 5);
-
-  g_value_init(value,
-	       G_TYPE_UINT);
-    
-  g_value_init(value + 1,
-	       G_TYPE_UINT);
-    
-  g_value_init(value + 2,
-	       G_TYPE_POINTER);
-
-  g_value_init(value + 3,
-	       G_TYPE_POINTER);
-
-  g_value_init(value + 4,
-	       G_TYPE_POINTER);
-
   audio = NULL;
 
   recall_container = NULL;
@@ -530,12 +486,6 @@ ags_fx_vst3_audio_notify_samplerate_callback(GObject *gobject,
 	       "buffer-size", &buffer_size,
 	       "samplerate", &samplerate,
 	       NULL);
-
-  g_value_set_uint(value,
-		   buffer_size);
-
-  g_value_set_uint(value + 1,
-		   samplerate);
   
   start_recall_channel = ags_recall_container_get_recall_channel(recall_container);
 
@@ -581,25 +531,10 @@ ags_fx_vst3_audio_notify_samplerate_callback(GObject *gobject,
 	  
 	  guint nth;
 
-	  if(channel_data->icomponent != NULL){
-//	    ags_vst_icomponent_destroy(channel_data->icomponent);
-	  }
-
 //	  g_message("ags_fx_vst3_audio_notify_samplerate_callback() - instantiate channel data");
 	  
-	  channel_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
-									     &n_params,
-									     &parameter_name, &value);
-	      
-	  channel_data->iedit_controller = g_value_get_pointer(value + 2);
-	  channel_data->iaudio_processor = g_value_get_pointer(value + 3);
-
-	  channel_data->iedit_controller_host_editing = g_value_get_pointer(value + 4);
-
-	  channel_data->icomponent_handler = ags_vst_component_handler_new();
-	    
-	  ags_vst_iedit_controller_set_component_handler(channel_data->iedit_controller,
-							 channel_data->icomponent_handler);
+	  ags_vst_process_context_set_samplerate(channel_data->process_context,
+						 (gdouble) samplerate);	  
 	}
 	
 	if(!is_live_instrument){
@@ -608,25 +543,10 @@ ags_fx_vst3_audio_notify_samplerate_callback(GObject *gobject,
 
 	    input_data = channel_data->input_data[k];
 
-	    if(input_data->icomponent != NULL){
-//	      ags_vst_icomponent_destroy(input_data->icomponent);
-	    }
-
 //	    g_message("ags_fx_vst3_audio_notify_samplerate_callback() - instantiate input data");
-	    
-	    input_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
-									     &n_params,
-									     &parameter_name, &value);
-
-	    input_data->iedit_controller = g_value_get_pointer(value + 2);
-	    input_data->iaudio_processor = g_value_get_pointer(value + 3);
-
-	    input_data->iedit_controller_host_editing = g_value_get_pointer(value + 4);
-
-	    input_data->icomponent_handler = ags_vst_component_handler_new();
-	    
-	    ags_vst_iedit_controller_set_component_handler(input_data->iedit_controller,
-							   input_data->icomponent_handler);
+	  
+	    ags_vst_process_context_set_samplerate(input_data->process_context,
+						   (gdouble) samplerate);	  
 	  }
 	}
       }
@@ -664,8 +584,6 @@ ags_fx_vst3_audio_notify_samplerate_callback(GObject *gobject,
   if(recall_container != NULL){
     g_object_unref(recall_container);
   }
-
-  g_strfreev(parameter_name);
 }
 
 void
@@ -679,14 +597,7 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
   AgsVst3Plugin *vst3_plugin;
 
   GList *start_recall_channel, *recall_channel;
-
-#if HAVE_GLIB_2_68
-  GStrvBuilder *strv_builder;
-#endif
-  
-  gchar **parameter_name;
-    
-  guint n_params;
+      
   guint input_pads;
   guint output_port_count, input_port_count;
   guint buffer_size;
@@ -694,8 +605,6 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
   guint i, j, k;
   gboolean is_live_instrument;
 
-  GValue *value;
-
   GRecMutex *recall_mutex;
 
   /* get recall mutex */
@@ -720,52 +629,6 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
     return;
   }
 
-#if HAVE_GLIB_2_68
-  strv_builder = g_strv_builder_new();
-
-  g_strv_builder_add(strv_builder,
-		     "buffer-size");
-  g_strv_builder_add(strv_builder,
-		     "samplerate");
-  g_strv_builder_add(strv_builder,
-		     "iedit-controller");
-  g_strv_builder_add(strv_builder,
-		     "iaudio-processor");
-  g_strv_builder_add(strv_builder,
-		     "iedit-controller-host-editing");
-  
-  parameter_name = g_strv_builder_end(strv_builder);
-#else
-  parameter_name = (gchar **) g_malloc(6 * sizeof(gchar *));
-
-  parameter_name[0] = g_strdup("buffer-size");
-  parameter_name[1] = g_strdup("samplerate");
-  parameter_name[2] = g_strdup("iedit-controller");
-  parameter_name[3] = g_strdup("iaudio-processor");
-  parameter_name[4] = g_strdup("iedit-controller-host-editing");
-  parameter_name[5] = NULL;
-#endif
-
-  n_params = 5;
-
-  value = g_new0(GValue,
-		 5);
-
-  g_value_init(value,
-	       G_TYPE_UINT);
-  
-  g_value_init(value + 1,
-	       G_TYPE_UINT);
-    
-  g_value_init(value + 2,
-	       G_TYPE_POINTER);
-
-  g_value_init(value + 3,
-	       G_TYPE_POINTER);
-
-  g_value_init(value + 4,
-	       G_TYPE_POINTER);
-
   start_input = NULL;
 
   input_pads = 0;
@@ -787,12 +650,6 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
 	       "samplerate", &samplerate,
 	       NULL);
   
-  g_value_set_uint(value,
-		   buffer_size);
-
-  g_value_set_uint(value + 1,
-		   samplerate);
-
   start_recall_channel = ags_recall_container_get_recall_channel(recall_container);
 
   /* get VST3 plugin */
@@ -846,21 +703,7 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
 	    guint nth;
 
 //	    g_message("ags_fx_vst3_audio_set_audio_channels_callback() - instantiate channel data");
-	    
-	    channel_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
-									       &n_params,
-									       &parameter_name, &value);
-	      
-	    channel_data->iedit_controller = g_value_get_pointer(value + 2);
-	    channel_data->iaudio_processor = g_value_get_pointer(value + 3);
-
-	    channel_data->iedit_controller_host_editing = g_value_get_pointer(value + 4);
-
-	    channel_data->icomponent_handler = ags_vst_component_handler_new();
-	    
-	    ags_vst_iedit_controller_set_component_handler(channel_data->iedit_controller,
-							   channel_data->icomponent_handler);
-	    
+	    	    
 	    ags_vst_process_data_set_num_samples(channel_data->process_data,
 						 buffer_size);
 	  
@@ -892,21 +735,7 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
 	      input_data = channel_data->input_data[k];
 	    
 //	      g_message("ags_fx_vst3_audio_set_audio_channels_callback() - instantiate input data");
-	      
-	      input_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
-									       &n_params,
-									       &parameter_name, &value);
-
-	      input_data->iedit_controller = g_value_get_pointer(value + 2);
-	      input_data->iaudio_processor = g_value_get_pointer(value + 3);
-
-	      input_data->iedit_controller_host_editing = g_value_get_pointer(value + 4);
-	      
-	      input_data->icomponent_handler = ags_vst_component_handler_new();
-	    
-	      ags_vst_iedit_controller_set_component_handler(input_data->iedit_controller,
-							     input_data->icomponent_handler);
-	      
+	      	      
 	      ags_vst_process_data_set_num_samples(input_data->process_data,
 						   buffer_size);
 	      
@@ -1120,6 +949,8 @@ ags_fx_vst3_audio_channel_data_alloc()
 {
   AgsFxVst3AudioChannelData *channel_data;
 
+  AgsVstIParameterChanges *iparameter_changes;
+  AgsVstIEventList *ievent_list;
   AgsVstAudioBusBuffers *output, *input;
   
   guint i;
@@ -1168,20 +999,34 @@ ags_fx_vst3_audio_channel_data_alloc()
   ags_vst_process_data_set_num_outputs(channel_data->process_data,
 				       1);
 
+  /* process context */
+  channel_data->process_context = ags_vst_process_context_alloc();
+
+  ags_vst_process_data_set_process_context(channel_data->process_data,
+					   channel_data->process_context);
+  
   /* parameter changes */
   channel_data->input_parameter_changes = ags_vst_parameter_changes_new();
 
   ags_vst_parameter_changes_set_max_parameters(channel_data->input_parameter_changes,
 					       AGS_FX_VST3_AUDIO_MAX_PARAMETER_CHANGES);
+
+  iparameter_changes = NULL;
+  ags_vst_funknown_query_interface(channel_data->input_parameter_changes,
+				   ags_vst_iparameter_changes_get_iid(), &iparameter_changes);
   
   ags_vst_process_data_set_input_iparameter_changes(channel_data->process_data,
-						    channel_data->input_parameter_changes);
+						    iparameter_changes);
   
   /* event list */
   channel_data->input_event = ags_vst_event_list_new();
 
+  ievent_list = NULL;
+  ags_vst_funknown_query_interface(channel_data->input_event,
+				   ags_vst_ievent_list_get_iid(), &ievent_list);
+
   ags_vst_process_data_set_input_events(channel_data->process_data,
-					channel_data->input_event);
+					ievent_list);
 
   /* input */
   input = ags_vst_audio_bus_buffers_alloc();
@@ -1267,6 +1112,8 @@ ags_fx_vst3_audio_input_data_alloc()
 {
   AgsFxVst3AudioInputData *input_data;
 
+  AgsVstIParameterChanges *iparameter_changes;
+  AgsVstIEventList *ievent_list;
   AgsVstAudioBusBuffers *output, *input;
   
 //  g_message("ags_fx_vst3_audio_input_data_alloc()");
@@ -1311,22 +1158,36 @@ ags_fx_vst3_audio_input_data_alloc()
   ags_vst_process_data_set_num_outputs(input_data->process_data,
 				       1);
 
+  /* process context */
+  input_data->process_context = ags_vst_process_context_alloc();
+
+  ags_vst_process_data_set_process_context(input_data->process_data,
+					   input_data->process_context);
+
   /* parameter changes */
   input_data->input_parameter_changes = ags_vst_parameter_changes_new();
 
   ags_vst_parameter_changes_set_max_parameters(input_data->input_parameter_changes,
 					       AGS_FX_VST3_AUDIO_MAX_PARAMETER_CHANGES);
 
+  iparameter_changes = NULL;
+  ags_vst_funknown_query_interface(input_data->input_parameter_changes,
+				   ags_vst_iparameter_changes_get_iid(), &iparameter_changes);
+
   ags_vst_process_data_set_input_iparameter_changes(input_data->process_data,
-						    input_data->input_parameter_changes);
+						    iparameter_changes);
 
   input_data->parameter_changes[0].param_id = ~0;
 
   /* event list */
   input_data->input_event = ags_vst_event_list_new();
 
+  ievent_list = NULL;
+  ags_vst_funknown_query_interface(input_data->input_event,
+				   ags_vst_ievent_list_get_iid(), &ievent_list);
+
   ags_vst_process_data_set_input_events(input_data->process_data,
-					input_data->input_event);
+					ievent_list);
 
   /* input */
   input = ags_vst_audio_bus_buffers_alloc();
@@ -1502,21 +1363,12 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
 {
   AgsVst3Manager *vst3_manager;
   AgsVst3Plugin *vst3_plugin;
-
-#if HAVE_GLIB_2_68
-  GStrvBuilder *strv_builder;
-#endif
-  
-  gchar **parameter_name;
   
   gchar *filename, *effect;
 
-  guint n_params;
   guint buffer_size;
   guint samplerate;
   
-  GValue *value;
-  
   GRecMutex *recall_mutex;
 
   if(!AGS_IS_FX_VST3_AUDIO(fx_vst3_audio)){
@@ -1525,52 +1377,6 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
 
   vst3_manager = ags_vst3_manager_get_instance();
 
-#if HAVE_GLIB_2_68
-  strv_builder = g_strv_builder_new();
-
-  g_strv_builder_add(strv_builder,
-		     "buffer-size");
-  g_strv_builder_add(strv_builder,
-		     "samplerate");
-  g_strv_builder_add(strv_builder,
-		     "iedit-controller");
-  g_strv_builder_add(strv_builder,
-		     "iaudio-processor");
-  g_strv_builder_add(strv_builder,
-		     "iedit-controller-host-editing");
-  
-  parameter_name = g_strv_builder_end(strv_builder);
-#else
-  parameter_name = (gchar **) g_malloc(6 * sizeof(gchar *));
-
-  parameter_name[0] = g_strdup("buffer-size");
-  parameter_name[1] = g_strdup("samplerate");
-  parameter_name[2] = g_strdup("iedit-controller");
-  parameter_name[3] = g_strdup("iaudio-processor");
-  parameter_name[4] = g_strdup("iedit-controller-host-editing");
-  parameter_name[5] = NULL;
-#endif
-  
-  n_params = 5;
-
-  value = g_new0(GValue,
-		 5);
-
-  g_value_init(value,
-	       G_TYPE_UINT);
-    
-  g_value_init(value + 1,
-	       G_TYPE_UINT);
-  
-  g_value_init(value + 2,
-	       G_TYPE_POINTER);
-
-  g_value_init(value + 3,
-	       G_TYPE_POINTER);
-
-  g_value_init(value + 4,
-	       G_TYPE_POINTER);
-
   /* get recall mutex */
   recall_mutex = AGS_RECALL_GET_OBJ_MUTEX(fx_vst3_audio);
   
@@ -1588,12 +1394,6 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
 	       "samplerate", &samplerate,
 	       NULL);
 
-  g_value_set_uint(value,
-		   buffer_size);
-
-  g_value_set_uint(value + 1,
-		   samplerate);
-
   /* check if already loaded */
   g_rec_mutex_lock(recall_mutex);
 
@@ -1640,21 +1440,7 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
 
 	  if(is_live_instrument){
 	    if(channel_data->icomponent == NULL){
-//	      g_message("ags_fx_vst3_audio_load_plugin() - instantiate channel data");
-	      
-	      channel_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
-										 &n_params,
-										 &parameter_name, &value);
-	      
-	      channel_data->iedit_controller = g_value_get_pointer(value + 2);
-	      channel_data->iaudio_processor = g_value_get_pointer(value + 3);
-
-	      channel_data->iedit_controller_host_editing = g_value_get_pointer(value + 4);
-
-	      channel_data->icomponent_handler = ags_vst_component_handler_new();
-	    
-	      ags_vst_iedit_controller_set_component_handler(channel_data->iedit_controller,
-							     channel_data->icomponent_handler);
+//	      g_message("ags_fx_vst3_audio_load_plugin() - instantiate channel data");	      
 	    }
 	  }
 	
@@ -1666,20 +1452,6 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
 
 	      if(input_data->icomponent == NULL){
 //		g_message("ags_fx_vst3_audio_load_plugin() - instantiate input data");
-		
-		input_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
-										 &n_params,
-										 &parameter_name, &value);
-
-		input_data->iedit_controller = g_value_get_pointer(value + 2);
-		input_data->iaudio_processor = g_value_get_pointer(value + 3);
-
-		input_data->iedit_controller_host_editing = g_value_get_pointer(value + 4);
-
-		input_data->icomponent_handler = ags_vst_component_handler_new();
-	    
-		ags_vst_iedit_controller_set_component_handler(input_data->iedit_controller,
-							       input_data->icomponent_handler);
 	      }
 	    }
 	  }
@@ -1688,12 +1460,8 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
     }
     
     g_rec_mutex_unlock(recall_mutex);
-
-    g_free(value);
   }
 
-  g_strfreev(parameter_name);
-  
   g_free(filename);
   g_free(effect);
 }
@@ -2086,27 +1854,38 @@ ags_fx_vst3_audio_safe_write_callback(AgsPort *port, GValue *value,
 				      AgsFxVst3Audio *fx_vst3_audio)
 {
   AgsAudio *audio;
-  AgsPluginPort *plugin_port;
-  
-  AgsVstParameterInfo *info;
-  
+  AgsChannel *start_output, *output;
+  AgsPlaybackDomain *playback_domain;
+
+  AgsVst3Plugin *vst3_plugin;
+
+  GList *start_output_playback, *output_playback;
+    
   guint audio_channels;
   gint sound_scope;
-  guint port_index;
-  gfloat param_value;
-  guint i;
+  guint i, i_start, i_stop;
+  guint j;
   gboolean is_live_instrument;
 
   GRecMutex *fx_vst3_audio_mutex;
   
   fx_vst3_audio_mutex = AGS_RECALL_GET_OBJ_MUTEX(fx_vst3_audio);
 
+  g_rec_mutex_lock(fx_vst3_audio_mutex);
+
+  vst3_plugin = fx_vst3_audio->vst3_plugin;
+  
+  g_rec_mutex_unlock(fx_vst3_audio_mutex);
+
+  if(vst3_plugin == NULL){
+    g_warning("can't get VST3 plugin");
+
+    return;
+  }
+
   audio = NULL;
-  plugin_port = NULL;
 
   audio_channels = 0;
-
-  port_index = 0;
   
   g_object_get(fx_vst3_audio,
 	       "audio", &audio,
@@ -2117,163 +1896,139 @@ ags_fx_vst3_audio_safe_write_callback(AgsPort *port, GValue *value,
 		 "audio-channels", &audio_channels,
 		 NULL);
   }
-
-  g_object_get(port,
-	       "plugin-port", &plugin_port,
-	       NULL);
-
-  if(plugin_port != NULL){
-    g_object_get(plugin_port,
-		 "port-index", &port_index,
-		 NULL);
-  }
     
   is_live_instrument = ags_fx_vst3_audio_test_flags(fx_vst3_audio, AGS_FX_VST3_AUDIO_LIVE_INSTRUMENT);
 
-  info = ags_vst_parameter_info_alloc();      
-  
-  g_rec_mutex_lock(fx_vst3_audio_mutex);
-  
-  for(sound_scope = 0; sound_scope < AGS_SOUND_SCOPE_LAST; sound_scope++){
-    AgsFxVst3AudioScopeData *audio_scope_data;
-
-    guint channel;
+  if(!is_live_instrument){  
+    g_critical("invalid port access of instrument");
     
-    audio_scope_data = fx_vst3_audio->scope_data[sound_scope];
+    if(audio != NULL){
+      g_object_unref(audio);
+    }
+    
+    return;
+  }
 
-    if(sound_scope == AGS_SOUND_SCOPE_PLAYBACK ||
-       sound_scope == AGS_SOUND_SCOPE_NOTATION ||
-       sound_scope == AGS_SOUND_SCOPE_MIDI){
-      for(channel = 0; channel < audio_channels; channel++){
-	AgsFxVst3AudioChannelData *channel_data;
+  /* get some fields */
+  playback_domain = NULL;
+  start_output = NULL;
+  
+  start_output_playback = NULL;
+  
+  g_object_get(audio,
+	       "playback-domain", &playback_domain,
+	       "output", &start_output,
+	       NULL);
+    
+  g_object_get(playback_domain,
+	       "output-playback", &start_output_playback,
+	       NULL);
 
-	guint key;
+  if(ags_base_plugin_test_flags((AgsBasePlugin *) vst3_plugin, AGS_BASE_PLUGIN_IS_INSTRUMENT)){
+    AgsAudioThread *audio_thread;
+    AgsChannelThread *channel_thread;
+
+    AgsTaskLauncher *task_launcher;
+
+    i_start = 0;
+    i_stop = audio_channels;
+    
+    for(i = i_start; i < i_stop && i < audio_channels; i++){
+      for(j = 0; j < AGS_SOUND_SCOPE_LAST; j++){
+	AgsWriteVst3Port *write_vst3_port;
+
+	audio_thread = ags_playback_domain_get_audio_thread(playback_domain,
+							    j);
+
+	output = ags_channel_nth(start_output,
+				 i);
 	
-	channel_data = audio_scope_data->channel_data[channel];
+	output_playback = start_output_playback;
+	
+	while(output_playback != NULL){
+	  AgsChannel *current_output;
 
-	if(is_live_instrument){
-	  AgsVstParamID param_id;
+	  gboolean success;
 
-	  if(channel_data->iedit_controller != NULL){
-	    ags_vst_iedit_controller_get_parameter_info(channel_data->iedit_controller,
-							port_index, info);
-
-	    param_id = ags_vst_parameter_info_get_param_id(info);
-	  	  
-	    param_value = ags_vst_iedit_controller_plain_param_to_normalized(channel_data->iedit_controller,
-									     param_id,
-									     g_value_get_double(value));
-	    
-#if 1
-	    if(channel_data->iedit_controller_host_editing != NULL){
-	      ags_vst_iedit_controller_host_editing_begin_edit_from_host(channel_data->iedit_controller_host_editing,
-									 param_id);
-	    }
-	    
-	    ags_vst_iedit_controller_set_param_normalized(channel_data->iedit_controller,
-							  param_id, param_value);
-
-	    for(i = 0; i < AGS_FX_VST3_AUDIO_MAX_PARAMETER_CHANGES; i++){
-	      if(fx_vst3_audio->parameter_changes[i].param_id == ~0){
-		fx_vst3_audio->parameter_changes[i].param_id = param_id;
-		fx_vst3_audio->parameter_changes[i].param_value = g_value_get_double(value);
-		
-		if(i +  1 < AGS_FX_VST3_AUDIO_MAX_PARAMETER_CHANGES){
-		  fx_vst3_audio->parameter_changes[i + 1].param_id = ~0;
-		}
-		
-		break;
-	      }
-	    }
-	    
-	    if(channel_data->iedit_controller_host_editing != NULL){
-	      ags_vst_iedit_controller_host_editing_end_edit_from_host(channel_data->iedit_controller_host_editing,
-								       param_id);
-	    }
-#else
-	    ags_vst_icomponent_handler_begin_edit(channel_data->icomponent_handler,
-						  param_id);
-
-	    ags_vst_icomponent_handler_perform_edit(channel_data->icomponent_handler,
-						    param_id, param_value);
-      
-	    ags_vst_icomponent_handler_end_edit(channel_data->icomponent_handler,
-						param_id);
-#endif
-	  }
-	}else{	
-	  for(key = 0; key < AGS_SEQUENCER_MAX_MIDI_KEYS; key++){
-	    AgsFxVst3AudioInputData *input_data;
+	  success = FALSE;
 	  
-	    AgsVstParamID param_id;
+	  g_object_get(output_playback->data,
+		       "channel", &current_output,
+		       NULL);
 
-	    input_data = channel_data->input_data[key];
+	  if(output == current_output){
+	    success = TRUE;
+	  }
 
-	    if(input_data->iedit_controller != NULL){
-	      ags_vst_iedit_controller_get_parameter_info(input_data->iedit_controller,
-							  port_index, info);
+	  if(current_output != NULL){
+	    g_object_unref(current_output);
+	  }
+	  
+	  if(success){
+	    break;
+	  }
+	  
+	  output_playback = output_playback->next;
+	}
 
-	      param_id = ags_vst_parameter_info_get_param_id(info);
+	channel_thread = NULL;
 
-	      param_value = ags_vst_iedit_controller_plain_param_to_normalized(input_data->iedit_controller,
-									       param_id,
-									       g_value_get_double(value));
-	    
-#if 1
-	      if(input_data->iedit_controller_host_editing != NULL){
-		ags_vst_iedit_controller_host_editing_begin_edit_from_host(input_data->iedit_controller_host_editing,
-									   param_id);
-	      }
-	    
-	      ags_vst_iedit_controller_set_param_normalized(input_data->iedit_controller,
-							    param_id, param_value);
+	if(output_playback != NULL){
+	  channel_thread = ags_playback_get_channel_thread(output_playback->data,
+							   j);
+	}
 
-	      
-	      for(i = 0; i < AGS_FX_VST3_AUDIO_MAX_PARAMETER_CHANGES; i++){
-		if(fx_vst3_audio->parameter_changes[i].param_id == ~0){
-		  fx_vst3_audio->parameter_changes[i].param_id = param_id;
-		  fx_vst3_audio->parameter_changes[i].param_value = g_value_get_double(value);
-		
-		  if(i +  1 < AGS_FX_VST3_AUDIO_MAX_PARAMETER_CHANGES){
-		    fx_vst3_audio->parameter_changes[i + 1].param_id = ~0;
-		  }
-		
-		  break;
-		}
-	      }
-	      
-	      if(input_data->iedit_controller_host_editing != NULL){
-		ags_vst_iedit_controller_host_editing_end_edit_from_host(input_data->iedit_controller_host_editing,
-									 param_id);
-	      }
-#else
-	      ags_vst_icomponent_handler_begin_edit(input_data->icomponent_handler,
-						    param_id);
-
-	      ags_vst_icomponent_handler_perform_edit(input_data->icomponent_handler,
-						      param_id, param_value);
-      
-	      ags_vst_icomponent_handler_end_edit(input_data->icomponent_handler,
-						  param_id);
-#endif	      
+	task_launcher = NULL;
+	
+	if(ags_playback_domain_test_flags(playback_domain, AGS_PLAYBACK_DOMAIN_SUPER_THREADED_AUDIO)){
+	  if(!ags_playback_test_flags(output_playback->data, AGS_PLAYBACK_SUPER_THREADED_CHANNEL)){
+	    if(audio_thread != NULL){
+	      g_object_get(audio_thread,
+			   "task-launcher", &task_launcher,
+			   NULL);
+	    }
+	  }else{
+	    if(channel_thread != NULL){
+	      g_object_get(channel_thread,
+			   "task-launcher", &task_launcher,
+			   NULL);
 	    }
 	  }
+	}else{
+	  //NOTE:JK: wtf!
+	}
+	
+	if(task_launcher != NULL){
+	  write_vst3_port = ags_write_vst3_port_new(fx_vst3_audio,
+						    port,
+						    g_value_get_double(value),
+						    j,
+						    i);
+
+	  ags_task_launcher_add_task(task_launcher,
+				     write_vst3_port);
+
+	  g_object_unref(task_launcher);
+	}
+	
+	if(output != NULL){
+	  g_object_unref(output);
 	}
       }
     }
-  }
+  }  
 
-  g_rec_mutex_unlock(fx_vst3_audio_mutex);
-
-  ags_vst_parameter_info_free(info);
-  
+  /* unref */
   if(audio != NULL){
     g_object_unref(audio);
   }
 
-  if(plugin_port != NULL){
-    g_object_unref(plugin_port);
+  if(start_output != NULL){
+    g_object_unref(start_output);
   }
+  
+  g_list_free_full(start_output_playback,
+		   (GDestroyNotify) g_object_unref);
 }
 
 /**
